use Test::More tests => 1;
BEGIN { use_ok( 'POE::Component::SmokeBox::Recent' ); };
diag( "Testing POE::Component::SmokeBox::Recent-$POE::Component::SmokeBox::Recent::VERSION, POE-$POE::VERSION, Perl $], $^X" );
